package com.topcoder.db.generation.xsd2ddl.renderers;

import java.io.IOException;
import java.io.OutputStream;

import com.topcoder.db.generation.xsd2ddl.DefinitionRenderer;
import com.topcoder.db.generation.xsd2ddl.RenderException;
import com.topcoder.db.generation.xsd2ddl.data.Database;

/**
 * This class in an implementation of the DefinitionRenderer interface that produces a
 * DROP script for Oracle 10g corresponding to the output produced by OracleCreateRenderer.
 *
 * The rendering process takes as input a Database object that contains instances of all the
 * other classes from the data package (Field, FieldType, Table, TableType, Key).
 *
 * The destination of the rendering in an output stream. The content that is written to the output
 * are Oracle 10g specific DROP statements. The defined entities (tables, constraints) are dropped
 * in the right order, corresponding to how they were created. 
 * 
 * Note:
 * 
 * This class has a constructor taking a namespace but the namespace is not really used for anything.
 * There is also a default constructor.
 * 
 * Thread safety:
 * 
 * This class is thread safe because it has no state.
 * 
 * @author TCSDESIGNER
 */
public class OracleDropRenderer implements DefinitionRenderer {
	
	/**
	 * Constructor. Does nothing.
	 * 
	 * @param namespace the configuration namespace
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all space
	 */
	public OracleDropRenderer(String namespace) throws IllegalArgumentException {
	}

	/**
	 * Default constructor. Does nothing.
	 */
	public OracleDropRenderer() {
	}

	/**
	 * Renders the database definition from input Database object (can include
	 * Field, FileType, Key, Table, TableType instances) to a stream.
	 * 
	 * The output is a Oracle 10g compatible DROP script that corresponds to the
	 * DDL script generated by OracleCreateRenderer
	 * 
	 * Implementation details:
	 * 
	 * - for each table in database
	 * 
	 * - for each field of the table
	 * - counter = 1
	 * - iterate all field type constraints in turn (those with value different than -1 or null)
	 * - output "ALTER TABLE " + table.getTableName() + " DROP CONSTRAINT " + tableName + "_" + fieldName + "_" + (counter++) + ";\r\n" 
	 * - end for each field
	 * 
	 * - for each foreign key
	 * - output "ALTER TABLE " + table.getTableName() + " DROP FOREIGN KEY " + ket.getKeyName() + ";\r\n"
	 * - end for each foreign key

	 * - if table has primary key
	 * - output "ALTER TABLE " + table.getTableName() + " DROP PRIMARY KEY " + key.getKeyName() + ";\r\n"
	 * - end if
	 * 
	 * - output "DROP TABLE " + table.getTableName() + ";\r\n"
	 * 
	 * - end for each table
	 * 
	 * @param definition the database definition
	 * @param renderStream the destination stream
	 * 
	 * @throws IllegalArgumentException if any argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void render(Database definition, OutputStream renderStream) throws IllegalArgumentException, IOException, RenderException {
	}

}
